//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#pragma once
#include "types.h"
#include "opcodes.h"
#include "otherfunctions.h"
#include "MD5Sum.h"
#include <string>

using namespace std;

#pragma pack(1)
// DO NOT EDIT VALUES like making a uint16 to uint32, or insert any value. ONLY append new vars
struct Preferences_Ext_Struct{
	int8	version;
	uchar	userhash[16];
	WINDOWPLACEMENT EmuleWindowPlacement;
};
// Maverick
struct ProxySettings{
	uint16 type;
	uint16 port;
	char name[50];
	char user[50];
	char password[50];
	bool EnablePassword;
	bool UseProxy;
};

enum SecuredVarErrors
{
	VariableNotInitialized
};

// Secured variable and array
template <class T> class CSecuredVar
{
public:
	CSecuredVar()
	{
		m_bInitialized = false;
	}

	~CSecuredVar()
	{
	}

	T Get()
	{
		if(!m_bInitialized)
			throw VariableNotInitialized;
		CSingleLock AccessLock(&m_Access, TRUE);
		T TempVal = m_Val;
		return TempVal;
	}

	void Put(T NewVal)
	{
		CSingleLock AccessLock(&m_Access, TRUE);
		m_bInitialized = true;
		m_Val = NewVal;
	}

private:
	T m_Val;

	CMutex	m_Access;
	bool	m_bInitialized;
};
/*
template <class T, unsigned int nSize> class CSecuredArr
{
public:
	CSecuredVar()
	{
		m_bInitialized = false;
	}

	~CSecuredVar()
	{
	}

	T* GetPointer()
	{
		if(!m_bInitialized)
			throw VariableNotInitialized;

		return m_pVal;
	}

	void Put(T* pNewVal)
	{
		CSingleLock AccessLock(&m_Access, TRUE);
		m_bInitialized = true;
		memcpy(m_pVal, pNewVal, nSize);
	}

	int Compare(T* pCompareVal)
	{
		CSingleLock AccessLock(&m_Access, TRUE);
		int nRet = memcmp(m_pVal, pCompareVal, nSize);
		return nRet;
	}

private:
	T m_pVal[nSize];

	CMutex	m_Access;
	bool	m_bInitialized;
};
*/

typedef enum
{
	SEE_SHARE_EVERYBODY = 0,
	SEE_SHARE_FRIENDS = 1,
	SEE_SHARE_NOONE = 2
} enSeeShare;

struct Preferences_Struct_DONTUSE{
//	char	nick[255];
//	uint16	maxupload;
//	uint16	maxdownload;
//	uint16	port;
//	uint16	udpport;
//	uint16	maxconnections;
//	int8	reconnect;
//	int8	deadserver;
//	int8	scorsystem;
//	char	incomingdir[510];
//	char	tempdir[510];
//	char	vlcdir[510];
	//int8	ICH;
//	int8	autoserverlist;
//	int8	updatenotify;
//	int8	mintotray;
//	int8	autoconnect;
//	int8	autoconnectstaticonly; // Barry
//	int8	addnewfilespaused;     // Barry
//	int8	depth3D;			   // Barry
//	int8	addServersFromConn;
	//int8	addserversfromclient;
//	int16	maxsourceperfile;
//	int16	trafficOMeterInterval;
//	int16	statsInterval;
//	uint16	statsColumnWidth; // FoRcHa
//	char	statsListExpanded[16];
	uchar	userhash[16];
	WINDOWPLACEMENT EmuleWindowPlacement;
//	int		maxGraphDownloadRate;
//	int		maxGraphUploadRate;
//	uint8	beepOnError;
//	uint8	confirmExit;
	int16	downloadColumnWidths[11];	//more columns
	BOOL	downloadColumnHidden[11];
	INT		downloadColumnOrder[11];
	int16	uploadColumnWidths[10];
	BOOL	uploadColumnHidden[10];
	INT		uploadColumnOrder[10];
	int16	queueColumnWidths[13];
	BOOL	queueColumnHidden[13];
	INT		queueColumnOrder[13];
	int16	searchColumnWidths[5];
	BOOL	searchColumnHidden[5];
	INT		searchColumnOrder[5];
	//--- xrmb:parttraffic ---
	//--- its not 9 anymore, its 11 ---
	int16	sharedColumnWidths[11];
	BOOL	sharedColumnHidden[11];
	INT	sharedColumnOrder[11];
	//--- :xrmb ---
	int16	serverColumnWidths[9];
	BOOL	serverColumnHidden[9];
	INT 	serverColumnOrder[9];
	//Cax2 irc columns
	int16	ircColumnWidths[3];
	BOOL	ircColumnHidden[3];
	INT		ircColumnOrder[3];
	DWORD	statcolors[15];
//	uint8	GraphRatio; // propstats Cax2

//	uint8	splashscreen;
//	uint8	filerBadIP;
//	uint8	onlineSig;

	//--- xrmb:partTrafficPrefs ---
	bool	m_pt_useit;			// Cax2
	uint8	m_upbar_st;			// 0=one info 1=2 infos 2=3infos
	uint8	m_upbar_cl;			// 0=rainbow style 1=more rainbow 2=downloadlist-like 3=girlie
	//--- :xrmb ---

//	uint64  totalDownloadedBytes;
//	uint64	totalUploadedBytes;
//	WORD	languageID;
//	int8	transferDoubleclick;
//	int8	m_iSeeShares;		// 0=everybody 1=friends only 2=noone
	int8	m_iToolDelayTime;	// tooltip delay time in seconds
	int8	bringtoforeground;
	int8	splitterbarPosition;
	uint16	deadserverretries;
	int16	maxsourceperfilesoft;
	int16	maxsourceperfileudp;
	uint8	flatbar;
	uint16  statsMax;
	int8	statsAverageMinutes;
	int16	queuesize ;			// SeeWen QUEUESIZE
	int16	remotequeuelimit;
	//End of patch
	uint16 maxupclientsallowed;
	uint16 minupclientsallowed;
	uint16 uploadclientdatarate;
	uint32 minrequesttime;
	uint8  badclientban;
	uint32 badclientbantime;
	// reCDVst (Load/Save Sources settings)
	uint32 slsrestoresourcestime;
	uint32 slsloadquantities;
	uint32 slsoutdated;
	uint32 slsmaxsourcesperfile;
	bool slsenable;
	// DonGato
	uint16	Profile;
	uint32	NotificationDisplayTime;
	uint32	NotificationFontSize;
	int16	PriorityHigh;
	int16	PriorityLow;
	bool	CommunityEnabled;
	char	CommunityString[100];
	bool	CommunityNoBan;
	bool	RestartWaiting;
	bool	LogToFile;
	bool	TransferFullChunks;
	int8    closeToTray;
	bool	DetailsOnClick;
	uint32	SrvConTimeout;
	bool	BugReport;
	bool	DisableXSUpTo;
	uint16	XSUpTo;
	bool	SlowComplete;
	uint16	SlowCompleteBlockSize;
	char	UsedFont[100];
	uint8	FontSize;
	int8	HashingPriority; // (0=Idle 1=Lower 2=Standard)
	bool	UploadParts;

	int8	showToolbarSpeedMeter; 
	int8    useDownloadAddNotifier;
	int8    useDownloadNotifier;
	int8    useChatNotifier;
	int8    useLogNotifier;	
	int8    useSoundInNotifier;
	int8    notifierPopsEveryChatMsg;
	int8	notifierImportantError;
	char    notifierSoundFilePath[510];

	char	m_sircserver[50];
	char	m_sircnick[30];
	char	m_sircchannamefilter[50];
	bool	m_bircaddtimestamp;
	bool	m_bircusechanfilter;
	uint16	m_iircchanneluserfilter;
	char	m_sircperformstring[255];
	bool	m_bircuseperform;
	bool	m_birclistonconnect;
	bool	m_bircacceptlinks;
	bool	m_bircignoreinfomessage;
	bool	m_bircignoreemuleprotoinfomessage;

	bool	m_bpreviewprio;
	bool	smartidcheck;
	uint8	smartidstate;
	bool	SafeConnect;
	bool	startMinimized;
	uint16	MaxConperFive;
	bool	useSort; // Cax2
	bool	useSrcSortCol2;
	int8	serverSortCol;
	int8	uploadSortCol;
	int8	queueSortCol;
	int8	searchSortCol;
	int8	ircSortCol;
	uint16	fileSortCol;
	int8	downloadSortCol;
	int8	srcSortCol1;
	int8	srcSortCol2;

	// Tarod
	int8	UAP;
	int8	DAP;
	int8	disablexs;
	int8	startdownloadpaused;
	int8	downloadpausedonclomplete;
	int8	remove_nns;
	uchar	A4AF_file_hash[16];

	bool	m_bVerbose;
	bool	m_bupdatequeuelist;
	bool	m_bmanualhighprio;
	bool	m_btransferfullchunks;
	int8	globalsearch; // Barry

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	int		tableSortItemDownload;
	int		tableSortItemUpload;
	int		tableSortItemQueue;
	int		tableSortItemSearch;
	int		tableSortItemShared;
	int		tableSortItemServer;
	bool	tableSortAscendingDownload;
	bool	tableSortAscendingUpload;
	bool	tableSortAscendingQueue;
	bool	tableSortAscendingSearch;
	bool	tableSortAscendingShared;
	bool	tableSortAscendingServer;
	char	TxtEditor[256];

	bool	m_multiple;	// EpicStruggle 
	char	toolbarsettings[256];	// reCDVst

	//Maverick
	bool	MlDonkeyCM;
	ProxySettings proxy;
};

struct Preferences_Import19c_Struct{
	int8	version;
	char	nick[50];
	uint16	maxupload;
	uint16	maxdownload;
	uint16	port;
	uint16	maxconnections;
	int8	reconnect;
	int8	deadserver;
	int8	scorsystem;
	char	incomingdir[510];
	char	tempdir[510];
	int8	ICH;
	int8	autoserverlist;
	int8	updatenotify;
	int8	mintotray;
	uchar	userhash[16];
	int8	autoconnect;
	int8	addServersFromConn;
};

struct Preferences_Import20a_Struct{
	int8	version;
	char	nick[50];
	uint16	maxupload;
	uint16	maxdownload;
	uint16	port;
	uint16	maxconnections;
	int8	reconnect;
	int8	deadserver;
	uint16	deadserverretries;
	int8	scorsystem;
	char	incomingdir[510];
	char	tempdir[510];
	int8	ICH;
	int8	autoserverlist;
	int8	updatenotify;
	int8	mintotray;
	uchar	userhash[16];
	int8	autoconnect;
	int8	addServersFromConn;
	int8	addserversfromclient;
	int16	maxsourceperfile;
	int16	trafficOMeterInterval;
	int32   totalDownloaded;
	int32	totalUploaded;
	int	maxGraphDownloadRate;
	int	maxGraphUploadRate;
	uint8	beepOnError;
	uint8	confirmExit;
	WINDOWPLACEMENT EmuleWindowPlacement;
	int transferColumnWidths[9];
	int serverColumnWidths[8];
	uint8	splashscreen;
	uint8	filerBadIP;
};

struct Preferences_Import20b_Struct{
	int8	version;
	char	nick[50];
	uint16	maxupload;
	uint16	maxdownload;
	uint16	port;
	uint16	maxconnections;
	int8	reconnect;
	int8	deadserver;
	int8	scorsystem;
	char	incomingdir[510];
	char	tempdir[510];
	int8	ICH;
	int8	autoserverlist;
	int8	updatenotify;
	int8	mintotray;
	uchar	userhash[16];
	int8	autoconnect;
	int8	addServersFromConn;
	int8	addserversfromclient;
	int16	maxsourceperfile;
	int16	trafficOMeterInterval;
	int32   totalDownloaded;	// outdated
	int32	totalUploaded;		// outdated
	int		maxGraphDownloadRate;
	int		maxGraphUploadRate;
	uint8	beepOnError;
	uint8	confirmExit;
	WINDOWPLACEMENT EmuleWindowPlacement;
	int transferColumnWidths[9];
	int serverColumnWidths[8];
	uint8	splashscreen;
	uint8	filerBadIP;
	int64   totalDownloadedBytes;
	int64	totalUploadedBytes;
};

class CPreferences{
public:
	enum Table {tableDownload, tableUpload, tableQueue, tableSearch, tableShared, tableServer, tableIrc};

	CPreferences();
	~CPreferences();

	CMutex	m_Pref_Mutex_Profile;
	CMutex	m_Pref_Mutex_ListenPort;
	CMutex	m_Pref_Mutex_UAP;
	CMutex	m_Pref_Mutex_DAP;
	CMutex	m_Pref_Mutex_XS;
	CMutex	m_Pref_Mutex_DownloadPaused;
	CMutex	m_Pref_Mutex_DownloadPausedOnComplete;
	CMutex	m_Pref_Mutex_RemoveNNS;
	CMutex	m_Pref_Mutex_Toolbar;
	CMutex	m_Pref_Mutex_AutoServerList;
	CMutex	m_Pref_Mutex_A4AFHash;
	CMutex	m_Pref_Mutex_BadClientTime;
	CMutex	m_Pref_Mutex_BadClientMinReqTime;
	CMutex	m_Pref_Mutex_BadClientMinReqNum;
	CMutex	m_Pref_Mutex_SLSRestoreTime;
	CMutex	m_Pref_Mutex_SLSLoadQuantities;
	CMutex	m_Pref_Mutex_SLSOutdated;
	CMutex	m_Pref_Mutex_SLS;
	CMutex	m_Pref_Mutex_MaxUploadClients;
	CMutex	m_Pref_Mutex_MinUploadClients;
	CMutex	m_Pref_Mutex_UploadClientRate;
	CMutex	m_Pref_Mutex_QueueSize;
	CMutex	m_Pref_Mutex_AddServersFromConn;
	CMutex	m_Pref_Mutex_StatsColWidth;
	CMutex	m_Pref_Mutex_StatsExpanded;
	CMutex	m_Pref_Mutex_TotalDownloaded;
	CMutex	m_Pref_Mutex_TotalUploaded;
	CMutex	m_Pref_Mutex_MaxUploadRate;
	CMutex	m_Pref_Mutex_MaxDownloadRate;
	CMutex	m_Pref_Mutex_MaxConnections;
	CMutex	m_Pref_Mutex_MaxSourcesPerFile;
	CMutex	m_Pref_Mutex_MaxSourcesPerFileUDP;
	CMutex	m_Pref_Mutex_DeadServerRetry;
	CMutex	m_Pref_Mutex_Multiple;
	CMutex	m_Pref_Mutex_LangID;
	CMutex	m_Pref_Mutex_DoubleClick;
	CMutex	m_Pref_Mutex_CanSeeShares;
	CMutex	m_Pref_Mutex_TooltipDelay;
	CMutex	m_Pref_Mutex_BringToFront;
	CMutex	m_Pref_Mutex_SplitterPos;
	CMutex	m_Pref_Mutex_StatsMax;
	CMutex	m_Pref_Mutex_StatsAvg;
	CMutex	m_Pref_Mutex_ToolbarMeter;
	CMutex	m_Pref_Mutex_CloseToTray;
	CMutex	m_Pref_Mutex_DetailsOnClick;
	CMutex	m_Pref_Mutex_DownloadNotifier;
	CMutex	m_Pref_Mutex_ChatNotifier;
	CMutex	m_Pref_Mutex_LogNotifier;
	CMutex	m_Pref_Mutex_SoundNotifier;
	CMutex	m_Pref_Mutex_ErrorNotifier;
	CMutex	m_Pref_Mutex_WavePathNotifier;
	CMutex	m_Pref_Mutex_IRCParams;
	CMutex	m_Pref_Mutex_StartMin;
	CMutex	m_Pref_Mutex_SmartIdCheck;
	CMutex	m_Pref_Mutex_SmartIdState;
	CMutex	m_Pref_Mutex_Verbose;
	CMutex	m_Pref_Mutex_UpdateQueueList;
	CMutex	m_Pref_Mutex_Depth3D;
	CMutex	m_Pref_Mutex_StatsColor;
	CMutex	m_Pref_Mutex_Sort;
	CMutex	m_Pref_Mutex_MaxConPer5;
	CMutex	m_Pref_Mutex_SafeConnect;
	CMutex	m_Pref_Mutex_PartTraffic;
	CMutex	m_Pref_Mutex_Community;
	CMutex	m_Pref_Mutex_Notification;
	CMutex	m_Pref_Mutex_Priority;
	CMutex	m_Pref_Mutex_RestartWaiting;
	CMutex	m_Pref_Mutex_LogToFile;
	CMutex	m_Pref_Mutex_FullChunks;
	CMutex	m_Pref_Mutex_XSUpTo;
	CMutex	m_Pref_Mutex_SrvConTimeout;
	CMutex	m_Pref_Mutex_SlowComplete;
	CMutex	m_Pref_Mutex_Font;
	CMutex	m_Pref_Mutex_HashPriority;
	CMutex	m_Pref_Mutex_UploadParts;
	CMutex	m_Pref_Mutex_WndPlacement;
	CMutex	m_Pref_Mutex_MlDonkey;
	CMutex	m_Pref_Mutex_Proxy;

	char*	GetAppDir()									{return appdir;}
	bool	Save();

	uint16	Profile()									{CSingleLock PrefLock(&m_Pref_Mutex_Profile,TRUE); uint16 nRet = prefs->Profile; return nRet; }
	void	SetProfile(uint16 uProfile)					{CSingleLock PrefLock(&m_Pref_Mutex_Profile,TRUE); prefs->Profile = uProfile; }
	int8	Score()										{ return m_nScoreSystem.Get(); }
	void	SetScore(int8 iScore)						{ m_nScoreSystem.Put(iScore); }
	bool	Reconnect()									{ return m_bReconnect.Get(); }
	void	SetReconnect(bool bReconnect)				{ m_bReconnect.Put(bReconnect); }
	int8	DeadServer()								{ return m_nDeadServer.Get(); }
	void	SetDeadServer(int8 iDead)					{ m_nDeadServer.Put(iDead); }
	CString	GetUserNick()								{ return CString(m_sNick.Get().c_str()); }
	void    SetUserNick(CString strNewNick)				{ m_sNick.Put(strNewNick.GetBuffer(0)); }
	uint16	GetPort()									{ return m_nPort.Get(); }
	void	SetPort(uint16 uPort)						{ m_nPort.Put(uPort); }
	
    //Maverick
    uint16	GetListenPort()								{CSingleLock PrefLock(&m_Pref_Mutex_ListenPort,TRUE); uint16 nRet; if (m_UseProxyListenPort) nRet = ListenPort; else nRet = GetPort(); return nRet; }
	void	SetListenPort(uint16 uPort)					{CSingleLock PrefLock(&m_Pref_Mutex_ListenPort,TRUE); ListenPort = uPort; m_UseProxyListenPort = true; }
	void	ResetListenPort()							{CSingleLock PrefLock(&m_Pref_Mutex_ListenPort,TRUE); ListenPort = 0; m_UseProxyListenPort = false; }
	
	uint16	GetUDPPort()								{ return m_nUDPPort.Get(); }
	void	SetUDPPort(uint16 uPort)					{ m_nUDPPort.Put(uPort); }
	CString	GetIncomingDir()							{ return CString(m_sIncomingDir.Get().c_str()); }
	void	SetIncomingDir(CString strDir)				{ m_sIncomingDir.Put(strDir.GetBuffer(0)); }
	CString	GetTempDir()								{ return CString(m_sTempDir.Get().c_str()); }
	void	SetTempDir(CString strDir)					{ m_sTempDir.Put(strDir.GetBuffer(0)); }
	CString	GetVlcDir()  								{ return CString(m_sVlcDir.Get().c_str()); }
	void    SetVlcDir(CString strDir)					{ m_sVlcDir.Put(strDir.GetBuffer(0)); }
	uchar*	GetUserHash()								{ return prefs->userhash; }
	uint16	GetMaxUpload()								{ return m_nMaxUpload.Get(); }

	// Added by Tarod
	bool	IsUAPEnabled()								{CSingleLock PrefLock(&m_Pref_Mutex_UAP,TRUE); bool bRet = prefs->UAP; return bRet; }
	void	SetUAPEnabled(bool bEnable)					{CSingleLock PrefLock(&m_Pref_Mutex_UAP,TRUE); prefs->UAP = bEnable; }
	bool	IsDAPEnabled()								{CSingleLock PrefLock(&m_Pref_Mutex_DAP,TRUE); bool bRet = prefs->DAP; return bRet; }
	void	SetDAPEnabled(bool bEnable)					{CSingleLock PrefLock(&m_Pref_Mutex_DAP,TRUE); prefs->DAP = bEnable; }
//	bool	ReconnectLowID() 							{CSingleLock PrefLock(&m_Pref_Mutex,TRUE); return prefs->reconnectlowid;}
	bool	IsDisabledXS()								{CSingleLock PrefLock(&m_Pref_Mutex_XS,TRUE); bool bRet = prefs->disablexs; return bRet; }
	void	SetDisabledXS(bool bDisable)				{CSingleLock PrefLock(&m_Pref_Mutex_XS,TRUE); prefs->disablexs = bDisable; }
	bool	StartDownloadPaused()						{CSingleLock PrefLock(&m_Pref_Mutex_DownloadPaused,TRUE); bool bRet = prefs->startdownloadpaused; return bRet; }
	void	SetStartDownloadPaused(bool bPaused)		{CSingleLock PrefLock(&m_Pref_Mutex_DownloadPaused,TRUE); prefs->startdownloadpaused = bPaused; }
	bool	DownloadPausedOnComplete()					{CSingleLock PrefLock(&m_Pref_Mutex_DownloadPausedOnComplete,TRUE); bool bRet = prefs->downloadpausedonclomplete; return bRet; }
	void	SetDownloadPausedOnComplete(bool bPaused)	{CSingleLock PrefLock(&m_Pref_Mutex_DownloadPausedOnComplete,TRUE); prefs->downloadpausedonclomplete = bPaused; }
	int		IsRemoveNNSEnabled() 						{CSingleLock PrefLock(&m_Pref_Mutex_RemoveNNS,TRUE); int nRet = prefs->remove_nns; return nRet; }
	void	SetRemoveNNSEnabled(int iEnable)			{CSingleLock PrefLock(&m_Pref_Mutex_RemoveNNS,TRUE); prefs->remove_nns = iEnable; }
	CString GetToolbarSettings() 						{CSingleLock PrefLock(&m_Pref_Mutex_Toolbar,TRUE); CString sRet = prefs->toolbarsettings; return sRet; } // reCDVst
	void	SetToolbarSettings(char* in) 				{CSingleLock PrefLock(&m_Pref_Mutex_Toolbar,TRUE); sprintf(prefs->toolbarsettings,"%s",in); } // reCDVst
	bool	AutoServerlist()							{ return m_bAutoServerList.Get(); }
	void	SetAutoServerlist(bool bAuto)				{ m_bAutoServerList.Put(bAuto); }
	uchar*	GetA4AFHash(void)							{CSingleLock PrefLock(&m_Pref_Mutex_A4AFHash,TRUE); return prefs->A4AF_file_hash; } // Tarod
	void	SetA4AFHash(uchar* file_hash)				{CSingleLock PrefLock(&m_Pref_Mutex_A4AFHash,TRUE); memcpy(prefs->A4AF_file_hash, file_hash, sizeof(prefs->A4AF_file_hash)); } // Tarod

	// WebServer
	uint16	GetHTTPPort()								{ return m_nWebPort.Get(); }
	void	SetHTTPPort(uint16 uPort)					{ m_nWebPort.Put(uPort); }
	CString	GetHTTPPass()								{ return CString(m_sWebPassword.Get().c_str()); }
	void	SetHTTPPass(CString strNewPass)				{ m_sWebPassword.Put(MD5Sum(strNewPass).GetHash().GetBuffer(0)); }
	bool	GetHTTPIsEnabled()							{ return m_bWebEnabled.Get(); }
	void	SetHTTPIsEnabled(bool bEnable)				{ m_bWebEnabled.Put(bEnable); }
	bool	GetWebUseGzip()								{ return m_bWebUseGzip.Get(); }
	void	SetWebUseGzip(bool bUse)					{ m_bWebUseGzip.Put(bUse); }
	int		GetWebPageRefresh()							{ return m_nWebPageRefresh.Get(); }
	void	SetWebPageRefresh(int nRefresh)				{ m_nWebPageRefresh.Put(nRefresh); }

	bool	UpdateNotify()								{ return m_bUpdateNotify.Get(); }
	bool	DoMinToTray()								{ return m_bMinToTray.Get(); }
	void	SetMinToTray(bool bMinToTray)				{ m_bMinToTray.Put(bMinToTray); }
	bool	DoAutoConnect() 							{ return m_bAutoConnect.Get(); }
	void    SetAutoConnect(bool bConnect)				{ m_bAutoConnect.Put(bConnect); }
	uint32	BadClientBanTime()							{CSingleLock PrefLock(&m_Pref_Mutex_BadClientTime,TRUE); uint32 nRet = prefs->badclientbantime; return nRet; }
	void	SetBadClientBanTime(uint32 uTime)			{CSingleLock PrefLock(&m_Pref_Mutex_BadClientTime,TRUE); prefs->badclientbantime = uTime; }
	uint32	BadClientMinRequestTime()					{CSingleLock PrefLock(&m_Pref_Mutex_BadClientMinReqTime,TRUE); uint32 nRet = prefs->minrequesttime; return nRet; }
	void	SetBadClientMinRequestTime(uint32 uTime)	{CSingleLock PrefLock(&m_Pref_Mutex_BadClientMinReqTime,TRUE); prefs->minrequesttime = uTime; }
	uint8	BadClientMinRequestNum()					{CSingleLock PrefLock(&m_Pref_Mutex_BadClientMinReqNum,TRUE); uint8 nRet = prefs->badclientban; return nRet; }
	void	SetBadClientMinRequestNum(uint8 uVal)		{CSingleLock PrefLock(&m_Pref_Mutex_BadClientMinReqNum,TRUE); prefs->badclientban = uVal; }

	// reCDVst (Load/Save Sources settings)
	uint32	RestoreSourcesTime()						{CSingleLock PrefLock(&m_Pref_Mutex_SLSRestoreTime,TRUE); uint32 nRet = prefs->slsrestoresourcestime; return nRet; }
	void	SetRestoreSourcesTime(uint32 uTime)			{CSingleLock PrefLock(&m_Pref_Mutex_SLSRestoreTime,TRUE); prefs->slsrestoresourcestime = uTime; }
	uint32	LoadQuantities()							{CSingleLock PrefLock(&m_Pref_Mutex_SLSLoadQuantities,TRUE); uint32 nRet = prefs->slsloadquantities; return nRet; }
	void 	SetLoadQuantities(uint32 uVal)				{CSingleLock PrefLock(&m_Pref_Mutex_SLSLoadQuantities,TRUE); prefs->slsloadquantities = uVal; }
	uint32	WhenSourcesOutdated()						{CSingleLock PrefLock(&m_Pref_Mutex_SLSOutdated,TRUE); uint32 nRet = prefs->slsoutdated; return nRet; }
	void	SetWhenSourcesOutdated(uint32 uVal)			{CSingleLock PrefLock(&m_Pref_Mutex_SLSOutdated,TRUE); prefs->slsoutdated	=	uVal;}
	uint32	MaxSourcesPerFile()							{ return prefs->slsmaxsourcesperfile; }
	bool	SLSEnable()									{CSingleLock PrefLock(&m_Pref_Mutex_SLS,TRUE); bool bRet = prefs->slsenable; return bRet; }
	void	SetSLSEnable(bool bEnable)					{CSingleLock PrefLock(&m_Pref_Mutex_SLS,TRUE); prefs->slsenable = bEnable; }

	uint16	MaximumUploadClients()						{CSingleLock PrefLock(&m_Pref_Mutex_MaxUploadClients,TRUE); uint16 nRet = prefs->maxupclientsallowed; return nRet; }
	void	SetMaximumUploadClients(uint16 uVal)		{CSingleLock PrefLock(&m_Pref_Mutex_MaxUploadClients,TRUE); prefs->maxupclientsallowed = uVal; }
	uint16	MinimumUploadClients()						{CSingleLock PrefLock(&m_Pref_Mutex_MinUploadClients,TRUE); uint16 nRet = prefs->minupclientsallowed; return nRet; }
	void	SetMinimumUploadClients(uint16 uVal)		{CSingleLock PrefLock(&m_Pref_Mutex_MinUploadClients,TRUE); prefs->minupclientsallowed = uVal; }
	uint16	UploadClientDatarate()						{CSingleLock PrefLock(&m_Pref_Mutex_UploadClientRate,TRUE); uint16 nRet = prefs->uploadclientdatarate; return nRet; }
	void	SetUploadClientDatarate(uint16 uVal)		{CSingleLock PrefLock(&m_Pref_Mutex_UploadClientRate,TRUE); prefs->uploadclientdatarate = uVal; }
	int16	GetQueueSize()								{CSingleLock PrefLock(&m_Pref_Mutex_QueueSize,TRUE); uint16 nRet = prefs->queuesize; return nRet; }
	void	SetQueueSize(int16 iSize)					{CSingleLock PrefLock(&m_Pref_Mutex_QueueSize,TRUE); prefs->queuesize =	iSize; }
	int16	GetRemoteQueueLimit()						{ return prefs->remotequeuelimit; }
	
	bool	GetAddServersFromConn()						{ return m_bAddServersFromConn.Get(); }
	void	SetAddServersFromConn(bool bAdd)			{ m_bAddServersFromConn.Put(bAdd); }
	//bool	AddServersFromClient()						{CSingleLock PrefLock(&m_Pref_Mutex,TRUE); return prefs->addserversfromclient;}
	//void	SetAddServersFromClient(bool bAdd)			{CSingleLock PrefLock(&m_Pref_Mutex,TRUE); prefs->addserversfromclient	=	bAdd;}
//	int8*	GetMinTrayPTR() 							{CSingleLock PrefLock(&m_Pref_Mutex,TRUE); return &prefs->mintotray;}
	uint16	GetTrafficOMeterInterval() 					{ return m_nTrafficOMeterInterval.Get(); }
	void	SetTrafficOMeterInterval(uint16 in) 		{ m_nTrafficOMeterInterval.Put(in); }
	uint16	GetStatsInterval() 							{ return m_nStatsInterval.Get(); }
	void	SetStatsInterval(uint16 in) 				{ m_nStatsInterval.Put(in); }
	uint16  GetStatsColumnWidth()						{ return m_nStatsColumnWidth.Get(); }
	uint16  GetDetailColumnWidth()						{ return m_nDetailColumnWidth.Get(); }
	void    SetStatsColumnWidth(uint16 in)				{ m_nStatsColumnWidth.Put(in); }
	void    SetDetailColumnWidth(uint16 in)				{ m_nDetailColumnWidth.Put(in); }
	CString	GetStatsListExpanded()						{ return CString(m_sStatsListExpanded.Get().c_str()); }
	void	SetStatsListExpanded(CString in)			{ m_sStatsListExpanded.Put(in.GetBuffer(0)); }
	void	Add2TotalDownloaded(uint64 in)				{ uint64 nSet = m_nTotalDownloadedBytes.Get(); nSet += in; m_nTotalDownloadedBytes.Put(nSet); }
	uint64  GetTotalDownloaded()						{ return m_nTotalDownloadedBytes.Get(); }
	void	Add2TotalUploaded(uint64 in) 				{ uint64 nSet = m_nTotalUploadedBytes.Get(); nSet += in; m_nTotalUploadedBytes.Put(nSet); }
	uint64	GetTotalUploaded()							{ return m_nTotalUploadedBytes.Get(); }
	bool	IsErrorBeepEnabled()						{ return m_bBeepOnError.Get(); }
	void	SetErrorBeepEnabled(bool bEnable)			{ m_bBeepOnError.Put(bEnable); }
//	void	SetConfirmExitEnabled(bool bEnable)			{CSingleLock PrefLock(&m_Pref_Mutex,TRUE); prefs->confirmExit=bEnable;}
	bool	UseSplashScreen()							{ return m_bSplashScreen.Get(); }
	void	SetUseSplashScreen(bool bUse)				{ m_bSplashScreen.Put(bUse); }
	bool	FilterBadIPs()								{ return m_bFilterBadIP.Get(); }
	void    SetFilterBadIPs(bool bFilter)				{ m_bFilterBadIP.Put(bFilter); }
	bool	IsOnlineSignatureEnabled()  				{ return m_bOnlineSig.Get(); }
	void	SetOnlineSignatureEnabled(bool bEnable)		{ m_bOnlineSig.Put(bEnable); }

	int		GetMaxGraphUploadRate()						{ return m_nMaxGraphUploadRate.Get(); }
	void	SetMaxGraphUploadRate(int in)				{ m_nMaxGraphUploadRate.Put(in); }
	int		GetMaxGraphDownloadRate()					{ return m_nMaxGraphDownloadRate.Get(); }
	void	SetMaxGraphDownloadRate(int in)				{ m_nMaxGraphDownloadRate.Put(in); }

	uint16	GetMaxDownload();
	uint16	GetMaxConnections()							{ return m_nMaxConnections.Get(); }
	void	SetMaxConnections(uint16 uMaxConn)			{ m_nMaxConnections.Put(uMaxConn); }
	uint16	GetMaxSourcePerFile()						{ return m_nMaxSourcePerFile.Get(); }
	void	SetMaxSourcePerFile(uint16 uVal)			{ m_nMaxSourcePerFile.Put(uVal); }
	uint16	GetMaxSourcePerFileSoft()					{CSingleLock PrefLock(&m_Pref_Mutex_MaxSourcesPerFile,TRUE); uint16 temp = m_nMaxSourcePerFile.Get()*0.9; if(temp > 1000) return 1000; return temp; }
	uint16  GetMaxSourcesPerFileSoftValue()				{CSingleLock PrefLock(&m_Pref_Mutex_MaxSourcesPerFile,TRUE); uint16 nRet = prefs->maxsourceperfilesoft; return nRet; }
	void    SetMaxSourcesPerFileSoftValue(uint16 uSrc)  {CSingleLock PrefLock(&m_Pref_Mutex_MaxSourcesPerFile,TRUE); prefs->maxsourceperfilesoft = uSrc; }
	uint16	GetMaxSourcePerFileUDP()					{CSingleLock PrefLock(&m_Pref_Mutex_MaxSourcesPerFile,TRUE); uint16 temp = m_nMaxSourcePerFile.Get()*0.75; if(temp > 100) return 100; return temp; }
	uint16  GetMaxSourcesPerFileUDPValue()				{CSingleLock PrefLock(&m_Pref_Mutex_MaxSourcesPerFileUDP,TRUE); uint16 nRet = prefs->maxsourceperfileudp; return nRet; }
	void	SetMaxSourcesPerFileUDPValue(uint16 uVal)	{CSingleLock PrefLock(&m_Pref_Mutex_MaxSourcesPerFileUDP,TRUE); prefs->maxsourceperfileudp = uVal; }
	uint16	GetDeadserverRetries()						{CSingleLock PrefLock(&m_Pref_Mutex_DeadServerRetry,TRUE); uint16 nRet = prefs->deadserverretries; return nRet; }
	void	SetDeadserverRetries(uint16 uRetries)		{CSingleLock PrefLock(&m_Pref_Mutex_DeadServerRetry,TRUE); prefs->deadserverretries	= uRetries; }

	int     GetColumnWidth (Table t, int index) const;
	BOOL    GetColumnHidden(Table t, int index) const;
	int     GetColumnOrder (Table t, int index) const;
	void	SetColumnWidth (Table t, int index, int width);
	void	SetColumnHidden(Table t, int index, BOOL bHidden);
	void	SetColumnOrder (Table t, INT *piOrder);

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	int		GetColumnSortItem (Table t) const;
	bool	GetColumnSortAscending (Table t) const;
	void	SetColumnSortItem (Table t, int sortItem);
	void	SetColumnSortAscending (Table t, bool sortAscending);



	bool	GetMultiple()								{CSingleLock PrefLock(&m_Pref_Mutex_Multiple,TRUE); bool bRet = prefs->m_multiple; return bRet; } //EpicStruggle
	void	SetMultiple(bool bEnable)					{CSingleLock PrefLock(&m_Pref_Mutex_Multiple,TRUE); prefs->m_multiple = bEnable; } //EpicStruggle

	WORD	GetLanguageID()								{ return m_nLanguageID.Get(); }
	void	SetLanguageID(WORD wID)						{ m_nLanguageID.Put(wID); }
	int8	IsDoubleClickEnabled()						{ return m_nTransferDoubleclick.Get(); }
	void	SetDoubleClickEnabled(int8 iEnable)			{ m_nTransferDoubleclick.Put(iEnable); }
	enSeeShare CanSeeShares(void)						{ return m_nSeeShares.Get(); }
	void	SetCanSeeShares(enSeeShare iCan)			{ m_nSeeShares.Put(iCan); }
	int8	GetToolTipDelay(void)						{CSingleLock PrefLock(&m_Pref_Mutex_TooltipDelay,TRUE); int8 nRet = prefs->m_iToolDelayTime; return nRet; }
	void	SetToolTipDelay(int8 iDelay)				{CSingleLock PrefLock(&m_Pref_Mutex_TooltipDelay,TRUE); prefs->m_iToolDelayTime	= iDelay; }
	int8	IsBringToFront()							{CSingleLock PrefLock(&m_Pref_Mutex_BringToFront,TRUE); int8 nRet = prefs->bringtoforeground; return nRet; }
	void	SetBringToFront(int8 bBTFront)				{CSingleLock PrefLock(&m_Pref_Mutex_BringToFront,TRUE); prefs->bringtoforeground = bBTFront; }

	int8    GetSplitterbarPosition()					{CSingleLock PrefLock(&m_Pref_Mutex_SplitterPos,TRUE); int8 nRet = prefs->splitterbarPosition; return nRet; }
	void	SetSplitterbarPosition(int8 pos) 			{CSingleLock PrefLock(&m_Pref_Mutex_SplitterPos,TRUE); prefs->splitterbarPosition = pos; }
	uint16	GetStatsMax()								{CSingleLock PrefLock(&m_Pref_Mutex_StatsMax,TRUE); uint16 nRet = prefs->statsMax; return nRet; }
	void	SetStatsMax(uint16 uMaxStats)				{CSingleLock PrefLock(&m_Pref_Mutex_StatsMax,TRUE); prefs->statsMax = uMaxStats; }
	int8	GetStatsAverageMinutes()					{CSingleLock PrefLock(&m_Pref_Mutex_StatsAvg,TRUE); int8 nRet = prefs->statsAverageMinutes; return nRet; }
	void	SetStatsAverageMinutes(int8 in)				{CSingleLock PrefLock(&m_Pref_Mutex_StatsAvg,TRUE); prefs->statsAverageMinutes = in; }

	bool	GetShowToolbarSpeedMeter()					{CSingleLock PrefLock(&m_Pref_Mutex_ToolbarMeter,TRUE); bool bRet = prefs->showToolbarSpeedMeter; return bRet; }
	void	SetShowToolbarSpeedMeter(bool bShow)		{CSingleLock PrefLock(&m_Pref_Mutex_ToolbarMeter,TRUE); prefs->showToolbarSpeedMeter = bShow; }
	bool    GetCloseToTray()							{CSingleLock PrefLock(&m_Pref_Mutex_CloseToTray,TRUE); bool bRet = prefs->closeToTray; return bRet; }
	void	SetCloseToTray(bool bClose)					{CSingleLock PrefLock(&m_Pref_Mutex_CloseToTray,TRUE); prefs->closeToTray = bClose; }
	bool    GetDetailsOnClick()							{CSingleLock PrefLock(&m_Pref_Mutex_DetailsOnClick,TRUE); bool bRet = prefs->DetailsOnClick; return bRet; }
	void    SetDetailsOnClick(bool bSet)				{CSingleLock PrefLock(&m_Pref_Mutex_DetailsOnClick,TRUE); prefs->DetailsOnClick = bSet; }
	bool    GetBugReport()								{ return prefs->BugReport; }
	bool    GetUseDownloadAddNotifier()					{CSingleLock PrefLock(&m_Pref_Mutex_DownloadNotifier,TRUE); bool bRet = prefs->useDownloadAddNotifier; return bRet; }
	void    SetUseDownloadAddNotifier(bool bUse)		{CSingleLock PrefLock(&m_Pref_Mutex_DownloadNotifier,TRUE); prefs->useDownloadAddNotifier = bUse; }
	bool    GetUseDownloadNotifier() 					{CSingleLock PrefLock(&m_Pref_Mutex_DownloadNotifier,TRUE); bool bRet = prefs->useDownloadNotifier; return bRet; }
	void    SetUseDownloadNotifier(bool bUse)			{CSingleLock PrefLock(&m_Pref_Mutex_DownloadNotifier,TRUE); prefs->useDownloadNotifier = bUse; }
	bool    GetUseChatNotifier()						{CSingleLock PrefLock(&m_Pref_Mutex_ChatNotifier,TRUE); bool bRet = prefs->useChatNotifier; return bRet; }
	void    SetUseChatNotifier(bool bUse)				{CSingleLock PrefLock(&m_Pref_Mutex_ChatNotifier,TRUE); prefs->useChatNotifier = bUse; }
	bool    GetUseLogNotifier()			 				{CSingleLock PrefLock(&m_Pref_Mutex_LogNotifier,TRUE); bool bRet = prefs->useLogNotifier; return bRet; }
	void    SetUseLogNotifier(bool bUse) 				{CSingleLock PrefLock(&m_Pref_Mutex_LogNotifier,TRUE); prefs->useLogNotifier = bUse; }
	bool    GetUseSoundInNotifier()		 				{CSingleLock PrefLock(&m_Pref_Mutex_SoundNotifier,TRUE); bool bRet = prefs->useSoundInNotifier; return bRet; }
	void    SetUseSoundInNotifier(bool bUse)			{CSingleLock PrefLock(&m_Pref_Mutex_SoundNotifier,TRUE); prefs->useSoundInNotifier = bUse; }
	bool    GetNotifierPopsEveryChatMsg()				{CSingleLock PrefLock(&m_Pref_Mutex_ChatNotifier,TRUE); bool bRet = prefs->notifierPopsEveryChatMsg; return bRet; }
	void    SetNotifierPopsEveryChatMsg(bool bPop)		{CSingleLock PrefLock(&m_Pref_Mutex_ChatNotifier,TRUE); prefs->notifierPopsEveryChatMsg = bPop; }
	bool	GetNotifierPopOnImportantError()			{CSingleLock PrefLock(&m_Pref_Mutex_ErrorNotifier,TRUE); bool bRet = prefs->notifierImportantError; return bRet; }
	void	SetNotifierPopOnImportantError(bool bPop)	{CSingleLock PrefLock(&m_Pref_Mutex_ErrorNotifier,TRUE); prefs->notifierImportantError = bPop; }
	CString GetNotifierWavSoundPath() 					{CSingleLock PrefLock(&m_Pref_Mutex_WavePathNotifier,TRUE); CString sRet = CString(prefs->notifierSoundFilePath); return sRet; }
	void	SetNotifierWavSoundPath(CString strPath)	{CSingleLock PrefLock(&m_Pref_Mutex_WavePathNotifier,TRUE); sprintf(prefs->notifierSoundFilePath,"%s",strPath); }

	CString	GetIRCNick()								{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); CString sRet = (CString)prefs->m_sircnick; return sRet; }
	void	SetIRCNick( char in_nick[] )				{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); strcpy(prefs->m_sircnick,in_nick); }
	CString	GetIRCServer()								{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); CString sRet = (CString)prefs->m_sircserver; return sRet; }
	void	SetIRCServer( char in_serv[] )				{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); strcpy(prefs->m_sircserver,in_serv); }
	bool	GetIRCAddTimestamp()						{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); bool bRet = prefs->m_bircaddtimestamp; return bRet; }
	void	SetIRCAddTimestamp( bool flag )				{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); prefs->m_bircaddtimestamp = flag; }
	CString	GetIRCChanNameFilter()						{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); CString sRet = (CString)prefs->m_sircchannamefilter; return sRet; }
	void	SetIRCChanNameFilter(char in_name[])		{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); strcpy(prefs->m_sircchannamefilter,in_name); }
	bool	GetIRCUseChanFilter()						{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); bool bRet = prefs->m_bircusechanfilter; return bRet; }
	void	SetIRCUseChanFilter(bool flag)				{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); prefs->m_bircusechanfilter = flag; }
	uint16	GetIRCChannelUserFilter()					{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); uint16 nRet = prefs->m_iircchanneluserfilter; return nRet; }
	void	SetIRCChanUserFilter(uint16 in_user)		{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); prefs->m_iircchanneluserfilter = in_user; }
	CString	GetIrcPerformString()						{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); return (CString)prefs->m_sircperformstring;}
	void	SetIRCPerformString(char in_perf[])			{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); strcpy(prefs->m_sircperformstring, in_perf);}
	bool	GetIrcUsePerform()							{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); return prefs->m_bircuseperform;}
	void	SetIrcUsePerform(bool flag)					{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); prefs->m_bircuseperform = flag;}
	bool	GetIRCListOnConnect()						{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); return prefs->m_birclistonconnect;}
	void	SetIRCListonConnect(bool flag)				{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); prefs->m_birclistonconnect = flag;}
	bool	GetIrcAcceptLinks()							{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); return prefs->m_bircacceptlinks;}
	void	SetIrcAcceptLInks(bool flag)				{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); prefs->m_bircacceptlinks = flag;}
	bool	GetIrcIgnoreInfoMessage()					{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); return prefs->m_bircignoreinfomessage;}
	void	SetIrcIgnoreInfoMessage(bool flag)			{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); prefs->m_bircignoreinfomessage = flag;}
	bool	GetIrcIgnoreEmuleProtoInfoMessage()			{CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); return prefs->m_bircignoreemuleprotoinfomessage;}
	void	SetIrcIgnoreEmuleProtoInfoMessage(bool flag){CSingleLock PrefLock(&m_Pref_Mutex_IRCParams,TRUE); prefs->m_bircignoreemuleprotoinfomessage = flag;}
	WORD	GetWindowsVersion();
	bool	GetStartMinimized()							{CSingleLock PrefLock(&m_Pref_Mutex_StartMin,TRUE); bool bRet = prefs->startMinimized; return bRet; }
	void    SetStartMinimized(bool bStart)				{CSingleLock PrefLock(&m_Pref_Mutex_StartMin,TRUE); prefs->startMinimized = bStart; }
	bool	GetSmartIdCheck()							{CSingleLock PrefLock(&m_Pref_Mutex_SmartIdCheck,TRUE); bool bRet = prefs->smartidcheck; return bRet; }
	void	SetSmartIdCheck(bool in_smartidcheck)		{CSingleLock PrefLock(&m_Pref_Mutex_SmartIdCheck,TRUE); prefs->smartidcheck = in_smartidcheck; }
	uint8	GetSmartIdState()							{CSingleLock PrefLock(&m_Pref_Mutex_SmartIdState,TRUE); uint8 nRet = prefs->smartidstate; return nRet; }
	void	SetSmartIdState(uint8 in_smartidstate)		{CSingleLock PrefLock(&m_Pref_Mutex_SmartIdState,TRUE); prefs->smartidstate = in_smartidstate; }

	bool	GetVerbose()						        {CSingleLock PrefLock(&m_Pref_Mutex_Verbose,TRUE); bool bRet =prefs->m_bVerbose; return bRet; }
	void 	SetVerbose(bool bEnabled)			        {CSingleLock PrefLock(&m_Pref_Mutex_Verbose,TRUE); prefs->m_bVerbose = bEnabled; }
	bool	GetPreviewPrio()					        { return prefs->m_bpreviewprio; }
	bool	GetUpdateQueueList()				        {CSingleLock PrefLock(&m_Pref_Mutex_UpdateQueueList,TRUE); bool bRet = prefs->m_bupdatequeuelist; return bRet; }
	void   SetUpdateQueueList(bool bEnabled)			{CSingleLock PrefLock(&m_Pref_Mutex_UpdateQueueList,TRUE); prefs->m_bupdatequeuelist = bEnabled; }
	bool	GetManualHighPrio()					        { return prefs->m_bmanualhighprio;}
	bool	ShowOverhead()						        { return m_bShowOverhead.Get();}
	void	SetShowOverhead(bool bit)				{ m_bShowOverhead.Put(bit);}
	bool	BackupPreview()								{return m_bBackupPreview.Get();}
	void	SetBackupPreview(bool bit)				{ m_bBackupPreview.Put(bit);}

//	char*	GetTxtEditor()						        {CSingleLock PrefLock(&m_Pref_Mutex,TRUE); return prefs->TxtEditor;}

	// Barry
	int8	UseFlatBar()								{ return m_nDepth3D.Get()==0; }
	int8	Get3DDepth()                                { return m_nDepth3D.Get(); }
	void	Set3DDepth(int8 iStyle)                      { m_nDepth3D.Put(iStyle); }
//	bool	AddNewFilesPaused()                         {CSingleLock PrefLock(&m_Pref_Mutex,TRUE); return prefs->addnewfilespaused;}
	bool    AutoConnectStaticOnly()                   	{return m_bAutoConnectStaticOnly.Get();}	
	void    SetAutoConnectStaticOnly(bool bEnabled)		{m_bAutoConnectStaticOnly.Put(bEnabled);}	

	DWORD	GetStatsColor(int index) 					{CSingleLock PrefLock(&m_Pref_Mutex_StatsColor,TRUE); DWORD dwRet = prefs->statcolors[index]; return dwRet; }
	void	SetStatsColor(int index,DWORD value) 		{CSingleLock PrefLock(&m_Pref_Mutex_StatsColor,TRUE); prefs->statcolors[index] = value; }
	uint8	GetGraphRatio()								{ return m_nGraphRatio.Get(); }
	void	SetGraphRatio(uint8 iNum)					{ m_nGraphRatio.Put(iNum); }
	bool	DoUseSort()									{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); bool bRet = prefs->useSort; return bRet; } //sortdlg Cax2
	void    SetUseSort(bool bUseSort)					{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->useSort = bUseSort; } //sortdlg Cax2
	bool	DoUseSrcSortCol2()							{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); bool bRet = prefs->useSrcSortCol2; return bRet; }
	void	SetUseSrcSortCol2(bool bUseSort)			{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->useSrcSortCol2 = bUseSort; }
	int8	GetServerSortCol()							{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); int8 nRet = prefs->serverSortCol; return nRet; }
	void	SetServerSortCol(int8 iCol)					{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->serverSortCol = iCol; }
	int8	GetUploadSortCol()							{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); int8 nRet = prefs->uploadSortCol; return nRet; }
	void	SetUploadSortCol(int8 iSortCol)				{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->uploadSortCol = iSortCol; }
	int8	GetQueueSortCol()							{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); int8 nRet = prefs->queueSortCol; return nRet; }
	void	SetQueueSortCol(int8 iSortCol)				{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->queueSortCol = iSortCol; }
	int8	GetSearchSortCol()							{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); int8 nRet = prefs->searchSortCol; return nRet; }
	void	SetSearchSortCol(int8 iSortCol)				{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->searchSortCol = iSortCol; }
	int8	GetIrcSortCol()								{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); int8 nRet = prefs->ircSortCol; return nRet; }
	void	SetIrcSortCol(int8 iSortCol)				{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->ircSortCol = iSortCol; }
	uint16	GetFileSortCol()							{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); uint16 nRet = prefs->fileSortCol; return nRet; }
	void	SetFileSortCol(uint16 uCol)					{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->fileSortCol = uCol; }
	int8	GetDownloadSortCol()						{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); int8 nRet = prefs->downloadSortCol; return nRet; }
	void	SetDownloadSortCol(int8 iSortCol)			{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->downloadSortCol = iSortCol; }
	int8	GetSrcSortCol1()							{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); int8 nRet = prefs->srcSortCol1; return nRet; }
	void	SetSrcSortCol1(int8 iSortCol)				{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->srcSortCol1 = iSortCol; }
	int8	GetSrcSortCol2()							{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); int8 nRet = prefs->srcSortCol2; return nRet; }
	void	SetSrcSortCol2(int8 iSortCol)				{CSingleLock PrefLock(&m_Pref_Mutex_Sort,TRUE); prefs->srcSortCol2 = iSortCol; }
	
	void	SetMaxDownloadConperFive(int in)			{CSingleLock PrefLock(&m_Pref_Mutex_MaxConPer5,TRUE); prefs->MaxConperFive = in; }
	uint16	GetMaxConperFive();
	uint16	GetDefaultMaxConperFive();

	//void	ResetStatsColor(int index); //Cax2 changed this function to DefaultStatsColor to use it in the colour picker!
	DWORD	DefaultStatsColor(int index);
	bool	IsSafeConnectEnabled()						{CSingleLock PrefLock(&m_Pref_Mutex_SafeConnect,TRUE); bool bRet = prefs->SafeConnect; return bRet; }
	void	SetSafeConnectEnabled(bool in)				{CSingleLock PrefLock(&m_Pref_Mutex_SafeConnect,TRUE); prefs->SafeConnect = in; }

	//--- xrmb:partTrafficPrefs ---
	bool	DoUsePT()									{CSingleLock PrefLock(&m_Pref_Mutex_PartTraffic,TRUE); bool bRet = prefs->m_pt_useit; return bRet; } // Cax2 05/01/03
	void	SetUsePT(bool in)							{CSingleLock PrefLock(&m_Pref_Mutex_PartTraffic,TRUE); prefs->m_pt_useit = in; }	// Cax2 05/01/03
	uint8	GetUpbarStyle()								{CSingleLock PrefLock(&m_Pref_Mutex_PartTraffic,TRUE); uint8 nRet = prefs->m_upbar_st; return nRet; }
	void	SetUpbarStyle(uint8 uStyle)					{CSingleLock PrefLock(&m_Pref_Mutex_PartTraffic,TRUE); prefs->m_upbar_st = uStyle; }
	uint8	GetUpbarColor()								{CSingleLock PrefLock(&m_Pref_Mutex_PartTraffic,TRUE); uint8 nRet = prefs->m_upbar_cl; return nRet; }
	void	SetUpbarColor(uint8 uColor)					{CSingleLock PrefLock(&m_Pref_Mutex_PartTraffic,TRUE); prefs->m_upbar_cl = uColor; }
	//--- :xrmb ---

	// DonGato
	bool	CommunityEnabled()							{CSingleLock PrefLock(&m_Pref_Mutex_Community,TRUE); bool bRet = prefs->CommunityEnabled; return bRet; }
	void	SetCommunityEnabled(bool bEnable)			{CSingleLock PrefLock(&m_Pref_Mutex_Community,TRUE); prefs->CommunityEnabled = bEnable; }
	CString	CommunityString()							{CSingleLock PrefLock(&m_Pref_Mutex_Community,TRUE); CString sRet = CString(prefs->CommunityString); return sRet; }
	void	SetCommunityString(CString strComm)			{CSingleLock PrefLock(&m_Pref_Mutex_Community,TRUE); strcpy(prefs->CommunityString,strComm); }
	bool	CommunityNoBanEnabled()						{CSingleLock PrefLock(&m_Pref_Mutex_Community,TRUE); bool bRet = prefs->CommunityNoBan; return bRet; }
	void	SetCommunityNoBanEnabled(bool bEnable)		{CSingleLock PrefLock(&m_Pref_Mutex_Community,TRUE); prefs->CommunityNoBan = bEnable; }
	uint32 	NotificationDisplayTime()					{CSingleLock PrefLock(&m_Pref_Mutex_Notification,TRUE); uint32 nRet = prefs->NotificationDisplayTime; return nRet; }
	void 	SetNotificationDisplayTime(uint32 uTime)	{CSingleLock PrefLock(&m_Pref_Mutex_Notification,TRUE); prefs->NotificationDisplayTime = uTime; }
	uint32 	NotificationFontSize()						{CSingleLock PrefLock(&m_Pref_Mutex_Notification,TRUE); uint32 nRet = prefs->NotificationFontSize; return nRet; }
	void 	SetNotificationFontSize(uint32 uFontSize)	{CSingleLock PrefLock(&m_Pref_Mutex_Notification,TRUE); prefs->NotificationFontSize = uFontSize; }
	int16 	PriorityHigh()								{CSingleLock PrefLock(&m_Pref_Mutex_Priority,TRUE); int16 nRet = prefs->PriorityHigh; return nRet; }
	void 	SetPriorityHigh(int16 iPrio)				{CSingleLock PrefLock(&m_Pref_Mutex_Priority,TRUE); prefs->PriorityHigh = iPrio; }
	int16 	PriorityLow()								{CSingleLock PrefLock(&m_Pref_Mutex_Priority,TRUE); int16 nRet = prefs->PriorityLow; return nRet; }
	void 	SetPriorityLow(int16 iPrio)					{CSingleLock PrefLock(&m_Pref_Mutex_Priority,TRUE); prefs->PriorityLow = iPrio; }
	bool	RestartWaiting()							{CSingleLock PrefLock(&m_Pref_Mutex_RestartWaiting,TRUE); bool bRet = prefs->RestartWaiting; return bRet; }
	void	SetRestartWaiting(bool bSet)				{CSingleLock PrefLock(&m_Pref_Mutex_RestartWaiting,TRUE); prefs->RestartWaiting = bSet; }
	bool	LogToFile()									{CSingleLock PrefLock(&m_Pref_Mutex_LogToFile,TRUE); bool bRet = prefs->LogToFile; return bRet; }
	void	SetLogToFile(bool bEnable)					{CSingleLock PrefLock(&m_Pref_Mutex_LogToFile,TRUE); prefs->LogToFile = bEnable; }
	bool	TransferFullChunks()						{CSingleLock PrefLock(&m_Pref_Mutex_FullChunks,TRUE); bool bRet = prefs->TransferFullChunks; return bRet; }
	void	SetTransferFullChunks(bool bEnable)			{CSingleLock PrefLock(&m_Pref_Mutex_FullChunks,TRUE); prefs->TransferFullChunks = bEnable; }
	bool	DisableXSUpTo()								{CSingleLock PrefLock(&m_Pref_Mutex_XSUpTo,TRUE); bool bRet = prefs->DisableXSUpTo; return bRet; }
	void	SetDisableXSUpTo(bool bDisable)				{CSingleLock PrefLock(&m_Pref_Mutex_XSUpTo,TRUE); prefs->DisableXSUpTo = bDisable; }
	uint16	XSUpTo()									{CSingleLock PrefLock(&m_Pref_Mutex_XSUpTo,TRUE); uint16 nRet = prefs->XSUpTo; return nRet; }
	void	SetXSUpTo(uint16 uVal)						{CSingleLock PrefLock(&m_Pref_Mutex_XSUpTo,TRUE); prefs->XSUpTo = uVal; }
	uint32  SrvConTimeout()								{CSingleLock PrefLock(&m_Pref_Mutex_SrvConTimeout,TRUE); uint32 nRet = prefs->SrvConTimeout; return nRet; }
	void	SetSrvConTimeout(uint32 timeout)			{CSingleLock PrefLock(&m_Pref_Mutex_SrvConTimeout,TRUE); prefs->SrvConTimeout = timeout; }
	bool	SlowComplete()								{CSingleLock PrefLock(&m_Pref_Mutex_SlowComplete,TRUE); bool bRet = prefs->SlowComplete; return bRet; }
	void	SetSlowComplete(uint16 bSlow)				{CSingleLock PrefLock(&m_Pref_Mutex_SlowComplete,TRUE); prefs->SlowComplete = bSlow; }
	uint16	SlowCompleteBlockSize()						{CSingleLock PrefLock(&m_Pref_Mutex_SlowComplete,TRUE); uint16 nRet = prefs->SlowCompleteBlockSize; return nRet; }
	void	SetSlowCompleteBlockSize(uint16 uSize)		{CSingleLock PrefLock(&m_Pref_Mutex_SlowComplete,TRUE); prefs->SlowCompleteBlockSize = uSize; }
	CString	GetUsedFont()								{CSingleLock PrefLock(&m_Pref_Mutex_Font,TRUE); CString sRet = (CString)prefs->UsedFont; return sRet; }
	void	SetUsedFont(CString font)					{CSingleLock PrefLock(&m_Pref_Mutex_Font,TRUE); strcpy(prefs->UsedFont,font); }
	uint8	GetFontSize()								{CSingleLock PrefLock(&m_Pref_Mutex_Font,TRUE); uint8 nRet = prefs->FontSize; return nRet; }
	void	SetFontSize(uint8 size)						{CSingleLock PrefLock(&m_Pref_Mutex_Font,TRUE); prefs->FontSize=size; }
	int8	GetHashingPriority(void)					{CSingleLock PrefLock(&m_Pref_Mutex_HashPriority,TRUE); int8 nRet = prefs->HashingPriority; return nRet; }
	void	SetHashingPriority(int8 hashing)			{CSingleLock PrefLock(&m_Pref_Mutex_HashPriority,TRUE); prefs->HashingPriority = hashing; }
	bool	IsUploadPartsEnabled()						{CSingleLock PrefLock(&m_Pref_Mutex_UploadParts,TRUE); bool bRet = prefs->UploadParts; return bRet; }
	void	SetUploadPartsEnabled(bool bEnable)			{CSingleLock PrefLock(&m_Pref_Mutex_UploadParts,TRUE); prefs->UploadParts = bEnable; }

	// quick-speed changer [xrmb]
	void	SetMaxUpload(uint16 nMaxUpload) 			{ m_nMaxUpload.Put(nMaxUpload); }
	void	SetMaxDownload(uint16 nMaxDownload)			{ m_nMaxDownload.Put(nMaxDownload); }
	uint16  GetMaxDownloadValue()						{ return m_nMaxDownload.Get(); }

	WINDOWPLACEMENT GetEmuleWindowPlacement()			{CSingleLock PrefLock(&m_Pref_Mutex_WndPlacement,TRUE); WINDOWPLACEMENT Ret = prefs->EmuleWindowPlacement; return Ret; }
	void SetWindowLayout(WINDOWPLACEMENT in) 			{CSingleLock PrefLock(&m_Pref_Mutex_WndPlacement,TRUE); prefs->EmuleWindowPlacement = in; }
	static int	GetRecommendedMaxConnections();

	void	LoadServerlistAddresses() ; // Tarod (read addresses.dat every time)

	// Maverick
	bool	GetMlDonkeyCM()								{CSingleLock PrefLock(&m_Pref_Mutex_MlDonkey,TRUE); bool bRet = prefs->MlDonkeyCM; return bRet; }
	void	SetMlDonkeyCM(bool bEnable)					{CSingleLock PrefLock(&m_Pref_Mutex_MlDonkey,TRUE); prefs->MlDonkeyCM = bEnable; }
	ProxySettings GetProxy()                            {CSingleLock PrefLock(&m_Pref_Mutex_Proxy,TRUE); ProxySettings Ret = prefs->proxy; return Ret; }
	void SetProxySettings(ProxySettings proxysettings)  {CSingleLock PrefLock(&m_Pref_Mutex_Proxy,TRUE); prefs->proxy = proxysettings; }

	CStringList shareddir_list;
	CStringList tempdir_list;	// InterCeptor (multiple tempdirs)
	CStringList adresses_list;

	void SetLanguage();
	int	GetIPFilterLevel()			{ return 128;}

protected:
	void		CreateUserHash();
	void		SetStandartValues();
private:
	char* appdir;
	Preferences_Struct_DONTUSE* prefs;
	Preferences_Ext_Struct* prefsExt;

	Preferences_Import19c_Struct* prefsImport19c;
	Preferences_Import20a_Struct* prefsImport20a;
	Preferences_Import20b_Struct* prefsImport20b;

	uchar userhash[16];
	WORD  m_wWinVer;

	//Maverick
    bool m_UseProxyListenPort;
	uint16	ListenPort;

	void LoadPreferences();
	void SavePreferences();

private:
	CSecuredVar<string>		m_sNick;
	CSecuredVar<WORD>		m_nLanguageID;

	CSecuredVar<uint16>		m_nMaxUpload;
	CSecuredVar<uint16>		m_nMaxDownload;
	CSecuredVar<uint16>		m_nMaxConnections;
	CSecuredVar<uint16>		m_nMaxSourcePerFile;

	CSecuredVar<uint64>		m_nTotalDownloadedBytes;
	CSecuredVar<uint64>		m_nTotalUploadedBytes;

	CSecuredVar<int>		m_nMaxGraphDownloadRate;
	CSecuredVar<int>		m_nMaxGraphUploadRate;
	CSecuredVar<uint8>		m_nGraphRatio;

	CSecuredVar<bool>		m_bReconnect;
	CSecuredVar<int8>		m_nDeadServer;
	CSecuredVar<int8>		m_nScoreSystem;
	CSecuredVar<bool>		m_bAutoServerList;
	CSecuredVar<bool>		m_bAutoConnect;
	CSecuredVar<bool>		m_bAutoConnectStaticOnly;
	CSecuredVar<bool>		m_bAddServersFromConn;
	CSecuredVar<bool>		m_bFilterBadIP;
	CSecuredVar<enSeeShare>	m_nSeeShares;

	CSecuredVar<bool>		m_bUpdateNotify;
	CSecuredVar<bool>		m_bMinToTray;
	CSecuredVar<bool>		m_bBeepOnError;
	CSecuredVar<bool>		m_bSplashScreen;
	CSecuredVar<bool>		m_bOnlineSig;
	CSecuredVar<bool>		m_bShowOverhead;
	CSecuredVar<bool>		m_bBackupPreview;
	CSecuredVar<int8>		m_nTransferDoubleclick;

	CSecuredVar<int8>		m_nDepth3D;
	CSecuredVar<uint16>		m_nTrafficOMeterInterval;
	CSecuredVar<uint16>		m_nStatsInterval;
	CSecuredVar<uint16>		m_nStatsColumnWidth;
	CSecuredVar<uint16>		m_nDetailColumnWidth;		//Cax2 
	CSecuredVar<string>		m_sStatsListExpanded;

	CSecuredVar<string>		m_sIncomingDir;
	CSecuredVar<string>		m_sTempDir;
	CSecuredVar<string>		m_sVlcDir;

	CSecuredVar<uint16>		m_nPort;
	CSecuredVar<uint16>		m_nUDPPort;

	// Web Server
	CSecuredVar<string>		m_sWebPassword;
	CSecuredVar<uint16>		m_nWebPort;
	CSecuredVar<bool>		m_bWebEnabled;
	CSecuredVar<bool>		m_bWebUseGzip;
	CSecuredVar<int>		m_nWebPageRefresh;
};
